@@ -61,8 +61,8 @@
 ;;   (set! calc-panels (list calc-arith-ops calc-trans-ops))
 ;;
 ;; This would give you only the artihmetic operations and some
-;; scientific functions.  There would be nothing else, even no digit
-;; pad.
+;; scientific functions.  There would be nothing else, not even a
+;; digit pad.
 
 ;; To define your own panels, you can use the "calc-panel" construct.
 ;; This is actually a macro, so the normal rules of argument
@@ -101,7 +101,7 @@
 ;; N-ARGS indicates how many arguments this function takes.  When
 ;; N-ARGS is a number, the calculator will pop this many items off its
 ;; stack and pass them to FUNC.  When N-ARGS is the symbol `all', the
-;; function will recieve the complete stack as its arguments.  Note:
+;; function will receive the complete stack as its arguments.  Note:
 ;; it will not be called with a single argument that is a list of all
 ;; stack items.  Rather it receives as many arguments as there are
 ;; stack items.  The first argument to FUNC is the last-popped stack
@@ -135,12 +135,12 @@
 ;; There is only one ROW, and the row contains only one BUTTON.  The
 ;; RETURN-ACTION of the BUTTON has been defaulted to `push'.
 ;;
-;; The `+' right after the "+" label is the Scheme variable the
+;; The `+' right after the "+" label is the Scheme variable that
 ;; carries the normal addition function.
 
-;; The next one is slightly more incolved.  It is a row of two button.
+;; The next one is slightly more involved.  It is a row of two buttons.
 ;; One swaps the two top elements of the stack, the other clears the
-;; stack. 
+;; stack.
 ;;
 ;;   (define my-stack-ops 
 ;;     (calc-panel (("swap" (lambda (a b) (list b a)) 2 push-list)
@@ -149,10 +149,10 @@
 ;; Note that we are using `lambda' here to construct the functions in
 ;; place.
 ;;
-;; `Swap' takes two arguments and pushes them and returns them in a
-;; list that will be pushed element-by-element.  `Clear' takes the
-;; whole stack as its arguments, but ignores it and pushes nothing,
-;; thus leaving the stack empty.
+;; `Swap' takes two arguments and returns them in a list that will be
+;; pushed element-by-element.  `Clear' takes the whole stack as its
+;; arguments, but ignores it and pushes nothing, thus leaving the
+;; stack empty.
 
 ;; For examples about the entry field, see the definition of
 ;; calc-digit-pad below.
@@ -163,7 +163,8 @@
 ;; We put the calculator into its own module to protect us from
 ;; overwriting builtin functions.
 
-(define-module (calc))
+(define-module (calc)
+  :use-module (toolkits gtk))
 
 ;; First some utility functions.  The first saves us from writing a
 ;; million times "gtk-widget-show".  It is like gtk-widget-show but
@@ -388,7 +389,7 @@
 ;; Anyway, all it does is to construct a Scheme expression that --
 ;; when evaluated -- in turn constructs a list that can be used as a
 ;; panel description.  A panel description is a list of rows, and a
-;; rows is a list of button specifications.  A button spec is a list
+;; row is a list of button specifications.  A button spec is a list
 ;; of four elements:
 ;;
 ;;   (LABEL FUNC N-ARGS RETURN-ACTION)
@@ -441,13 +442,13 @@
    (("asin" asin 1) ("acos" acos 1) ("atan" atan 1) ("log" log 1))
    (("sinh" sinh 1) ("cosh" cosh 1) ("tanh" tanh 1) ("expt" expt 2))))
 
-;; The stack operations are little bit more involved, but not really.
-;; Note that the "eval" button gives us a complete Scheme interpreter
-;; in our calculator.  (We should do some module magic here to protect
-;; us from evil evals.)  When RMS's visions come true about Guile
-;; supporting many popular syntaxes and semantics, we might get as
-;; easily a Phyton interpreter, or Perl, or TCL, or Java, or
-;; what-you-have.
+;; The stack operations are a little bit more involved, but not
+;; really.  Note that the "eval" button gives us a complete Scheme
+;; interpreter in our calculator.  (We should do some module magic
+;; here to protect us from evil evals.)  When RMS's visions come true
+;; about Guile supporting many popular syntaxes and semantics, we
+;; might get as easily a Phyton interpreter, or Perl, or TCL, or Java,
+;; or what-you-have.
 
 (define calc-stack-ops
   (calc-panel
@@ -496,7 +497,7 @@
     ("." (ins ".") entry set-entry)
     ("SPC" (ins " ") entry set-entry) ("+" + 2))))
 
-;; Predefine CALC-PANELS with some useful panels.  Not that the
+;; Predefine CALC-PANELS with some useful panels.  Note that the
 ;; arith-ops are not included because they are also on the digit-pad.
 
 (define calc-panels (list calc-stack-ops calc-trans-ops calc-digit-pad))
@@ -507,6 +508,4 @@
 (load-rcfile ".calcrc")
 (make-calculator calc-panels)
 
-;; Guile-gtk currently requires us to enter the event loop
-;; explicitely, maybe this will change.
 (gtk-main)
