@@ -17,6 +17,7 @@
 
 #include <assert.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkprivate.h>
 #include <libguile.h>
 #include <guile/gh.h>
 
@@ -64,17 +65,17 @@ static char *handle_key = "guile_scm_handle";
 static int
 gtkobj_print (SCM exp, SCM port, scm_print_state *pstate)
 {
-  scm_gen_puts (scm_regular_string, "#<", port);
+  scm_puts ("#<", port);
   if (GTKOBJ (exp) == NULL)
-    scm_gen_puts (scm_regular_string, "destroyed GtkObject", port);
+    scm_puts ("destroyed GtkObject", port);
   else
     {
       guint tid = GTK_OBJECT_TYPE (GTKOBJ (exp));
-      scm_gen_puts (scm_regular_string, gtk_type_name (tid), port);
-      scm_gen_puts (scm_regular_string, " ", port);
+      scm_puts (gtk_type_name (tid), port);
+      scm_puts (" ", port);
       scm_intprint ((long)GTKOBJ (exp), 16, port);
     }
-  scm_gen_puts (scm_regular_string, ">", port);
+  scm_puts (">", port);
   return 1;
 }
 
@@ -85,7 +86,11 @@ gtkobj_free (SCM obj)
   if (o)
     {
       if (o->ref_count == 0)
-	gtk_object_destroy (o);
+	{
+	  fprintf (stderr, "leaking %p (%s)\n",
+		   o, gtk_type_name (GTK_OBJECT_TYPE (o)));
+	  /* gtk_object_destroy (o); */
+	}
       else
 	gtk_object_remove_data (o, handle_key);
     }
@@ -362,12 +367,12 @@ static boxed *boxed_pool;
 static boxed *
 new_boxed ()
 {
-  // SCM_CRITICAL_SECTION_START;
+  /* SCM_CRITICAL_SECTION_START; */
   if (boxed_pool)
     {
       boxed *b = boxed_pool;
       boxed_pool = b->d.free_link;
-      // SCM_CRITICAL_SECTION_END;
+      /* SCM_CRITICAL_SECTION_END; */
       return b;
     }
   else
@@ -379,7 +384,7 @@ new_boxed ()
       for (i = 0; i < BOXED_CHUNK-2; i++, chunk++)
 	chunk->d.free_link = chunk+1;
       chunk->d.free_link = NULL;
-      // SCM_CRITICAL_SECTION_END;
+      /* SCM_CRITICAL_SECTION_END; */
       return chunk+1;
     }
 }
@@ -387,10 +392,10 @@ new_boxed ()
 static void
 free_boxed (boxed *b)
 {
-  // SCM_CRITICAL_SECTION_START;
+  /* SCM_CRITICAL_SECTION_START; */
   b->d.free_link = boxed_pool;
   boxed_pool = b;
-  // SCM_CRITICAL_SECTION_END;
+  /* SCM_CRITICAL_SECTION_END; */
 }
 
 #define BOXED_P(x)    (SCM_NIMP(x) && (SCM_CAR(x) == tc16_boxed))
@@ -409,11 +414,11 @@ boxed_free (SCM obj)
 static int
 boxed_print (SCM exp, SCM port, scm_print_state *pstate)
 {
-  scm_gen_puts (scm_regular_string, "#<", port);
-  scm_gen_puts (scm_regular_string, BOXED_INFO (exp)->name, port);
-  scm_gen_puts (scm_regular_string, " ", port);
+  scm_puts ("#<", port);
+  scm_puts (BOXED_INFO (exp)->name, port);
+  scm_puts (" ", port);
   scm_intprint ((long)BOXED_PTR (exp), 16, port);
-  scm_gen_puts (scm_regular_string, ">", port);
+  scm_puts (">", port);
   return 1;
 }
 
@@ -485,6 +490,36 @@ sgtk_find_boxed_info (GtkType type)
   return NULL;
 }
 
+sgtk_enum_info*
+sgtk_find_enum_info (GtkType type)
+{
+  /* XXX - use a hash or something */
+
+  char *name = gtk_type_name (type);
+  sgtk_enum_info *info;
+
+  for (info = sgtk_enum_infos; info->name; info++)
+    if (!strcmp (info->name, name))
+      return info;
+  SCM_ASSERT (0, SCM_BOOL_F, SCM_ARG1, "find-enum-info");
+  return NULL;
+}
+
+sgtk_enum_info*
+sgtk_find_flags_info (GtkType type)
+{
+  /* XXX - use a hash or something */
+
+  char *name = gtk_type_name (type);
+  sgtk_enum_info *info;
+
+  for (info = sgtk_flags_infos; info->name; info++)
+    if (!strcmp (info->name, name))
+      return info;
+  SCM_ASSERT (0, SCM_BOOL_F, SCM_ARG1, "find-flags-info");
+  return NULL;
+}
+
 /* Floats.
 
    Only here to set things straight. */
@@ -529,6 +564,9 @@ sgtk_arg2scm (GtkArg *a)
       return sgtk_boxed2scm (GTK_VALUE_BOXED(*a),
 			     sgtk_find_boxed_info (a->type),
 			     TRUE);
+    case GTK_TYPE_FLAGS:
+      return sgtk_flags2scm (GTK_VALUE_FLAGS(*a),
+			     sgtk_find_flags_info (a->type));
     default:
       fprintf (stderr, "illegal type %s in arg\n", 
 	       gtk_type_name (a->type));
@@ -627,6 +665,7 @@ SCM
 sgtk_color_conversion (SCM color)
 {
   SCM orig_color = color;
+  SCM_COERCE_SUBSTR (color);
 
   if (SCM_NIMP (color) && SCM_STRINGP (color))
     {
@@ -659,6 +698,7 @@ SCM
 sgtk_font_conversion (SCM font)
 {
   SCM orig_font = font;
+  SCM_COERCE_SUBSTR (font);
 
   if (SCM_NIMP (font) && SCM_STRINGP (font))
     {
@@ -670,6 +710,13 @@ sgtk_font_conversion (SCM font)
   return font;
 }
 
+SCM
+sgtk_string_conversion (SCM str)
+{
+  SCM_COERCE_SUBSTR (str);
+  return str;
+}
+
 
 
 /* Initialization */
@@ -687,5 +734,75 @@ sgtk_init (int *argcp, char ***argvp)
 
   sgtk_init_gtk_defs ();
 
-  gtk_init (argcp, argvp);
+  /* XXX - Initialize Gtk only once.  We assume that Gtk has already
+     been initialized when Gdk has.  That is not completely correct,
+     but the best I can do. */
+
+  if (gdk_display == NULL)
+    gtk_init (argcp, argvp);
+}
+
+void
+sgtk_init_argv (int argc, char **argv)
+{
+  sgtk_init (&argc, &argv);
+}
+
+static char*
+xstrdup (char *str)
+{
+  if (str)
+    {
+      char *newstr = scm_must_malloc (strlen(str)+1, "strdup");
+      strcpy (newstr, str);
+      return newstr;
+    }
+  else
+    return NULL;
+}
+
+static void
+make_argv (SCM list, int *argc, char ***argv)
+{
+  static char *argv_storage[1] = { "guile-gtk" };
+
+  int c = scm_ilength (list), i;
+  char **v;
+
+  *argv = argv_storage;
+  *argc = 1;
+
+  if (c < 0)
+    return;
+
+  v = (char **)scm_must_malloc ((c+1) * sizeof(char**), "make-argv");
+  for (i = 0; i < c; i++, list = SCM_CDR (list))
+    {
+      if (SCM_IMP (SCM_CAR (list)) || SCM_NSTRINGP (SCM_CAR (list)))
+	{
+	  scm_must_free ((char *)v);
+	  return;
+	}
+      v[i] = xstrdup (SCM_CHARS (SCM_CAR (list)));
+    }
+  v[c] = NULL;
+  
+  *argv = v;
+  *argc = c;
+}
+
+static void
+sgtk_init_noargs ()
+{
+  int argc;
+  char **argv;
+  make_argv (scm_program_arguments (), &argc, &argv);
+  sgtk_init (&argc, &argv);
+  scm_set_program_arguments (argc, argv, NULL);
+}
+
+void
+scm_init_toolkits_gtkstubs_module ()
+{
+  scm_register_module_xxx ("toolkits gtkstubs", sgtk_init_noargs);
 }
