@@ -123,6 +123,8 @@ parse_path (char *path)
 	ParsedPath *p = g_malloc (sizeof (ParsedPath));
 	char *sep;
 
+	g_assert(path != NULL);
+	
 	if (*path == '/' || prefix == NULL)
 		p->opath = strdup (path);
 	else
@@ -574,6 +576,8 @@ gnome_config_get_int (char *path)
 	pp = parse_path (path);
 	r = access_config (LOOKUP, pp->section, pp->key, pp->def, pp->file);
 
+	g_return_val_if_fail(r != NULL, 0);
+
 	if (!strcasecmp (r, "true")){
 		release_path (pp);
 		return 1;
