@@ -29,6 +29,8 @@ void gnome_config_clean_file     (char *path);
 /* This routine drops all of the information related to /file/section */
 void gnome_config_clean_section  (char *path);
 
+/* Drops the information for a specific key */
+void gnome_config_clean_key (char *path);
 void gnome_config_set_prefix (char *path);
 void gnome_config_drop_prefix (void);
 
