@@ -1,5 +1,5 @@
 /*
- * GMIX 1.5
+ * GMIX 2.0
  *
  * A program to control /dev/mixer* with a GTK+ UI.
  *
@@ -58,6 +58,12 @@ char *device_names[] = SOUND_DEVICE_NAMES;
 s_vol_info vol_info[SOUND_MIXER_NRDEVICES];
 s_vol m;
 
+/*
+ * The main window
+ */
+
+GtkWidget *window;
+
 void toggle_recsource (GtkWidget *widget, gint devnum)
 {
 	if (recmask & (1<<devnum)) {
@@ -129,14 +135,43 @@ void move_slider_r (GtkAdjustment *adjustment, s_vol_info *info)
 	if (info->mute_button!=NULL) gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(info->mute_button),FALSE);
 }
 
+void cleanup()
+{
+	int i;
+	gint x,y,w,h;
+	char cnf_string[255];
+	for (i=0;i<SOUND_MIXER_NRDEVICES;i++) {
+		if (devmask & (1<<i)) {
+			sprintf(cnf_string, "/gmix/%s/left", device_names[i]);
+			gnome_config_set_int(cnf_string, -(GTK_ADJUSTMENT(vol_info[i].adjustment_l))->value);
+			sprintf(cnf_string, "/gmix/%s/right", device_names[i]);
+			gnome_config_set_int(cnf_string, -(GTK_ADJUSTMENT(vol_info[i].adjustment_r))->value);
+			sprintf(cnf_string, "/gmix/%s/recsrc", device_names[i]);
+			gnome_config_set_int(cnf_string,  (recsrc & (1<<i)) ? 1 : 0);
+		}
+	}
+	/*
+	 * Save window position and size:
+	 */
+	gdk_window_get_origin(window->window, &x, &y);
+	gdk_window_get_size(window->window, &w, &h);
+
+	gnome_config_set_int("/gmix/geometry/width",w);
+	gnome_config_set_int("/gmix/geometry/height",h);
+	gnome_config_set_int("/gmix/geometry/xpos",x);
+	gnome_config_set_int("/gmix/geometry/ypos",y);
+	gnome_config_sync();
+	close(dev_mixer);
+}
+
 void destroy ()
 {
+	cleanup();
 	gtk_exit (0);
 }
 
 void create_dialog()
 {
-	GtkWidget *window;
 	GtkWidget *box;
 	GtkWidget *button;
 	GtkWidget *frame;
@@ -146,9 +181,17 @@ void create_dialog()
 	GtkWidget *scale;
 	GSList *group;
 	int i;
+	char cnf_string[255];
 
 	window=gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	gtk_widget_set_name (window, "G-MIX 1.0");
+	gtk_window_set_title (GTK_WINDOW(window), "G-MIX 2.0");
+	gtk_widget_set_usize(window, 
+		gnome_config_get_int("/gmix/geometry/width=-1"),
+		gnome_config_get_int("/gmix/geometry/height=200"));
+
+	gtk_widget_set_uposition(window, 
+		gnome_config_get_int("/gmix/geometry/xpos=-2"),
+		gnome_config_get_int("/gmix/geometry/ypos=-2"));
 	
 	box=gtk_hbox_new(FALSE, 5);
 	gtk_container_add(GTK_CONTAINER(window), box);
@@ -163,11 +206,15 @@ void create_dialog()
 			/* read current volume */
 			ioctl(dev_mixer, MIXER_READ(i), &m);
 
-			vol_info[i].adjustment_l=gtk_adjustment_new (-m.l, -101.0, 0.0, 1.0, 1.0, 0.0);
-			vol_info[i].adjustment_r=gtk_adjustment_new (-m.r, -101.0, 0.0, 1.0, 1.0, 0.0);
+			sprintf(cnf_string, "/gmix/%s/left=%i", device_names[i], m.l);
+			vol_info[i].adjustment_l=gtk_adjustment_new (-gnome_config_get_int(cnf_string), -101.0, 0.0, 1.0, 1.0, 0.0);
+
+			sprintf(cnf_string, "/gmix/%s/right=%i", device_names[i], m.r);
+			vol_info[i].adjustment_r=gtk_adjustment_new (-gnome_config_get_int(cnf_string), -101.0, 0.0, 1.0, 1.0, 0.0);
 		
 			/* draw control */
-			frame=gtk_frame_new(device_labels[i]);
+			sprintf(cnf_string, "/gmix/%s/titel=%s", device_names[i], device_labels[i]);
+			frame=gtk_frame_new(gnome_config_get_string(cnf_string));
 			gtk_box_pack_start_defaults(GTK_BOX(box), frame);
 			gtk_widget_show(frame);
 		
@@ -185,7 +232,6 @@ void create_dialog()
 				(GtkSignalFunc)move_slider_l,
 				(gpointer)&(vol_info[i]));
 			scale = gtk_vscale_new (GTK_ADJUSTMENT (vol_info[i].adjustment_l));
-			gtk_widget_set_usize (GTK_WIDGET (scale), 20, 150);
 			gtk_range_set_update_policy (GTK_RANGE (scale), GTK_UPDATE_CONTINUOUS);
 			gtk_scale_set_draw_value(GTK_SCALE(scale), FALSE);
 			gtk_box_pack_start (GTK_BOX (hbox), scale, TRUE, TRUE, 0);
@@ -198,7 +244,6 @@ void create_dialog()
 					(GtkSignalFunc)move_slider_r,
 					(gpointer)&(vol_info[i]));
 				scale = gtk_vscale_new (GTK_ADJUSTMENT (vol_info[i].adjustment_r));
-				gtk_widget_set_usize (GTK_WIDGET (scale), 20, 150);
 				gtk_range_set_update_policy (GTK_RANGE (scale), GTK_UPDATE_CONTINUOUS);
 				gtk_scale_set_draw_value(GTK_SCALE(scale), FALSE);
 				gtk_box_pack_start (GTK_BOX (hbox), scale, TRUE, TRUE, 0);
@@ -239,13 +284,16 @@ void create_dialog()
 	group=NULL;
 	for (i=0;i<SOUND_MIXER_NRDEVICES;i++) {
 		if (recmask & (1<<i)) {
+			sprintf(cnf_string, "/gmix/%s/titel=%s", device_names[i], device_labels[i]);
 			if (caps & SOUND_CAP_EXCL_INPUT) {
-				button = gtk_radio_button_new_with_label (group, device_names[i]);
+				button = gtk_radio_button_new_with_label (group, gnome_config_get_string(cnf_string));
 				group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
 			} else {
-				button = gtk_check_button_new_with_label (device_names[i]);
+				button = gtk_check_button_new_with_label (gnome_config_get_string(cnf_string));
 			}
-			if (recsrc & (1<<i)) 
+
+			sprintf(cnf_string, "/gmix/%s/recsrc=%i", device_names[i], (recsrc & (1<<i)) ? 1 : 0);
+			if (gnome_config_get_int(cnf_string)>0)
 				gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button),TRUE);
 			else
 				gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(button),FALSE);
@@ -269,7 +317,7 @@ void create_dialog()
 		GTK_SIGNAL_FUNC (gtk_widget_destroy),
 		GTK_OBJECT (window));
 	gtk_box_pack_end (GTK_BOX (vbox), button, FALSE, FALSE, 0);
-	gtk_widget_show (button);
+	gtk_widget_show(button);
 
 	gtk_widget_show(window);
 
@@ -326,12 +374,14 @@ void gerror(char *s)
 int main (int argc, char *argv[])
 {
 	int res;
+
 	gnome_init (&argc, &argv);
 
-	if (argc==2)
-		dev_mixer=open(argv[1], O_RDWR, 0);
-	else
-		dev_mixer=open("/dev/mixer", O_RDWR, 0);
+	if (argc==2) {
+		gnome_config_set_string ("/gmix/setup/device", argv[1]);
+	}
+	
+	dev_mixer=open(gnome_config_get_string("/gmix/setup/device=/dev/mixer"), O_RDWR, 0);
 
 	if (dev_mixer<0) {
 		gerror(argv[0]);
