@@ -8,50 +8,58 @@
  */
 
 #include <stdio.h>
+#ifdef HAVE_LIBINTL
 #include <libintl.h>
+#define _(String) gettext(String)
+#else
+#define _(String) (String)
+#endif
 #include <sys/stat.h>
 #include <unistd.h>
 #include <dirent.h>
 #include <string.h>
 #include "gnome.h"
-#include "../applet.h"
+#include "../panel_cmds.h"
+#include "../applet_cmds.h"
+#include "../panel.h"
+
+
+#define APPLET_ID "Mail check"
+
+
+static PanelCmdFunc panel_cmd_func;
 
 static char *mail_file;
 
+gpointer applet_cmd_func(AppletCommand *cmd);
+
 /* If set, the user has launched the mail viewer */
-int mailcleared;
+static int mailcleared;
 
 /* Does the user have any mail at all? */
-int anymail;
+static int anymail;
 
 /* New mail has arrived? */
-int newmail;
+static int newmail;
 
 /* Does the user have unread mail? */
-int unreadmail;
+static int unreadmail;
 
 /* The widget that holds the label with the mail information */
-GtkWidget *label;
+static GtkWidget *label;
 
 /* The drawing area */
-GtkWidget *da;
+static GtkWidget *da;
 
 /* handle for the timeout */
-int mail_timeout;
+static int mail_timeout;
 
-GtkWidget *pixmap_container;
-
-char *
-query (void)
-{
-	return "Mail check";
-}
 
 /*
  * Get file modification time, based upon the code
  * of Byron C. Darrah for coolmail and reused on fvwm95
  */
-void
+static void
 check_mail_file_status ()
 {
 	static off_t oldsize = 0;
@@ -78,7 +86,7 @@ check_mail_file_status ()
 	oldsize = newsize;
 }
 
-int
+static int
 mail_check_timeout (void *data)
 {
 	char *text;
@@ -93,18 +101,18 @@ mail_check_timeout (void *data)
 	return 1;
 }
 
-void
+static void
 mail_destroy (GtkWidget *widget, void *data)
 {
 	gtk_timeout_remove (mail_timeout);
 }
 
-GtkWidget *
+static GtkWidget *
 create_mail_widgets (GtkWidget *window)
 {
 	label = gtk_label_new ("");
 	da = gtk_drawing_area_new ();
-	gtk_drawing_area_size (da, 24, 20);
+	gtk_drawing_area_size ((GtkDrawingArea *) da, 24, 20);
 	
 	mail_check_timeout (0);
 	gtk_widget_show (label);
@@ -113,15 +121,8 @@ create_mail_widgets (GtkWidget *window)
 	return label;
 }
 
-void *
-mail_callback (AppletCommand *cmd)
-{
-	/* nothing now */
-	return 0;
-}
-
-void
-init (PanelCallback callback, Panel *panel, char *params, int xpos, int ypos)
+static void
+create_instance (Panel *panel, char *params, int xpos, int ypos)
 {
 	PanelCommand cmd;
 	GtkWidget *mailcheck;
@@ -136,13 +137,54 @@ init (PanelCallback callback, Panel *panel, char *params, int xpos, int ypos)
 
 	mailcheck = create_mail_widgets (panel->window);
 	cmd.cmd = PANEL_CMD_REGISTER_TOY;
-	cmd.params.register_toy.applet   = mailcheck;
-	cmd.params.register_toy.callback = mail_callback;
-	cmd.params.register_toy.id       = query();
-	cmd.params.register_toy.params   = params;
-	cmd.params.register_toy.xpos     = xpos;
-	cmd.params.register_toy.ypos     = ypos;
-	cmd.params.register_toy.flags    = 0;
+	cmd.params.register_toy.applet = mailcheck;
+	cmd.params.register_toy.id     = APPLET_ID;
+	cmd.params.register_toy.xpos   = xpos;
+	cmd.params.register_toy.ypos   = ypos;
+	cmd.params.register_toy.flags  = 0;
 
-	(*callback) (&cmd);
+	(*panel_cmd_func) (&cmd);
+}
+
+gpointer
+applet_cmd_func(AppletCommand *cmd)
+{
+	g_assert(cmd != NULL);
+
+	switch (cmd->cmd) {
+		case APPLET_CMD_QUERY:
+			return APPLET_ID;
+
+		case APPLET_CMD_INIT_MODULE:
+			panel_cmd_func = cmd->params.init_module.cmd_func;
+			break;
+
+		case APPLET_CMD_DESTROY_MODULE:
+			break;
+
+		case APPLET_CMD_GET_DEFAULT_PARAMS:
+			return g_strdup("");
+
+		case APPLET_CMD_CREATE_INSTANCE:
+			create_instance(cmd->panel,
+					cmd->params.create_instance.params,
+					cmd->params.create_instance.xpos,
+					cmd->params.create_instance.ypos);
+			break;
+
+		case APPLET_CMD_GET_INSTANCE_PARAMS:
+			return g_strdup("");
+
+		case APPLET_CMD_PROPERTIES:
+			fprintf(stderr, "Mail check properties\n"); /* FIXME */
+			break;
+
+		default:
+			fprintf(stderr,
+				APPLET_ID " applet_cmd_func: Oops, unknown command type %d\n",
+				(int) cmd->cmd);
+			break;
+	}
+
+	return NULL;
 }
