@@ -2,51 +2,60 @@
  * GNOME panel menu module.
  * (C) 1997 The Free Software Foundation
  *
- * Author: Miguel de Icaza
+ * Authors: Miguel de Icaza
+ *          Federico Mena
  */
 
 #include <stdio.h>
+#ifdef HAVE_LIBINTL
 #include <libintl.h>
+#define _(String) gettext(String)
+#else
+#define _(String) (String)
+#endif
 #include <sys/stat.h>
 #include <dlfcn.h>
 #include <unistd.h>
 #include <dirent.h>
 #include <string.h>
 #include "gnome.h"
-#include "../applet.h"
-#define _(String) gettext(String)
+#include "../panel_cmds.h"
+#include "../applet_cmds.h"
+#include "../panel.h"
 
 
-#define free_if_empty(x) { if (x) g_free (x); }
+#define APPLET_ID "Menu"
 
-char *gnome_folder;
+#define MENU_PATH "menu_path"
 
-PanelCallback callback;
 
-char *
-query (void)
-{
-	return "Menu";
-}
+static char *gnome_folder;
+
+static PanelCmdFunc panel_cmd_func;
+
+gpointer applet_cmd_func(AppletCommand *cmd);
+
+
+typedef struct {
+	char *translated;
+	char *original_id;
+} AppletItem;
+
 
 void
 activate_app_def (GtkWidget *widget, void *data)
 {
-	struct gnome_desktop_entry *item = data;
+	GnomeDesktopEntry *item = data;
 
 	gnome_desktop_entry_launch (item);
 }
 
 void
-setup_menuitem (GtkWidget *menuitem, GtkWidget *pixmap, char *thisfile)
+setup_menuitem (GtkWidget *menuitem, GtkWidget *pixmap, char *title)
 {
 	GtkWidget *label, *hbox, *align;
-	char *p = strstr (thisfile, ".desktop");
 
-	if (p)
-		*p = 0;
-
-	label = gtk_label_new (thisfile);
+	label = gtk_label_new (title);
 	gtk_misc_set_alignment (GTK_MISC(label), 0.0, 0.5);
 	gtk_widget_show (label);
 	
@@ -72,7 +81,7 @@ setup_menuitem (GtkWidget *menuitem, GtkWidget *pixmap, char *thisfile)
 void
 free_app_def (GtkWidget *widget, void *data)
 {
-	struct gnome_desktop_entry *item = data;
+	GnomeDesktopEntry *item = data;
 
 	gnome_desktop_entry_free (item);
 }
@@ -98,17 +107,19 @@ add_to_panel (char *applet, char *arg)
 {
 	PanelCommand cmd;
 	
-	fprintf (stderr, "Asking Panel to add [%s,%s] to the panel\n", applet, arg);
-	cmd.cmd = PANEL_CMD_NEW_APPLET;
-	cmd.params.new_applet.applet = applet;
-	cmd.params.new_applet.params  = arg;
-	(*callback) (&cmd);
+	cmd.cmd = PANEL_CMD_CREATE_APPLET;
+	cmd.params.create_applet.id     = applet;
+	cmd.params.create_applet.params = arg;
+	cmd.params.create_applet.xpos   = PANEL_UNKNOWN_APPLET_POSITION;
+	cmd.params.create_applet.ypos   = PANEL_UNKNOWN_APPLET_POSITION;
+
+	(*panel_cmd_func) (&cmd);
 }
 
 void
 add_app_to_panel (GtkWidget *widget, void *data)
 {
-	struct gnome_desktop_entry *ii = data;
+	GnomeDesktopEntry *ii = data;
 
 	add_to_panel ("Launcher", ii->location);
 }
@@ -116,13 +127,13 @@ add_app_to_panel (GtkWidget *widget, void *data)
 void
 add_dir_to_panel (GtkWidget *widget, void *data)
 {
-	add_to_panel (query (), data);
+	add_to_panel (APPLET_ID, data);
 }
 
 GtkWidget *
 create_menu_at (GtkWidget *window, char *menudir, int create_app_menu)
 {	
-	struct gnome_desktop_entry *item_info;
+	GnomeDesktopEntry *item_info;
 	GtkWidget *menu;
 	struct dirent *dent;
 	struct stat s;
@@ -140,6 +151,7 @@ create_menu_at (GtkWidget *window, char *menudir, int create_app_menu)
 		GtkWidget     *menuitem, *sub, *pixmap;
 		GtkSignalFunc activate_func;
 		char          *thisfile, *pixmap_name;
+		char          *p;
 
 		thisfile = dent->d_name;
 		/* Skip over . and .. */
@@ -168,7 +180,6 @@ create_menu_at (GtkWidget *window, char *menudir, int create_app_menu)
 				GtkWidget *pixmap = 0;
 				char *text;
 
-				/* FIXME: Loading the pixmap over and over is far from efficient */
 				text = g_copy_strings ("Menu: ", thisfile, NULL);
 
 				menuitem = gtk_menu_item_new ();
@@ -190,7 +201,6 @@ create_menu_at (GtkWidget *window, char *menudir, int create_app_menu)
 				gtk_signal_connect (GTK_OBJECT (menuitem), "destroy",
 						    (GtkSignalFunc) free_string,
 						    text);
-						    
 			}
 		} else {
 			if (strstr (filename, ".desktop") == 0){
@@ -216,6 +226,11 @@ create_menu_at (GtkWidget *window, char *menudir, int create_app_menu)
 			if (pixmap)
 				gtk_widget_show (pixmap);
 		}
+
+		p = strstr(thisfile, ".desktop");
+		if (p)
+			*p = '\0';  /* Remove the .desktop part */
+		
 		setup_menuitem (menuitem, pixmap, thisfile);
 
 		gtk_menu_append (GTK_MENU (menu), menuitem);
@@ -223,7 +238,7 @@ create_menu_at (GtkWidget *window, char *menudir, int create_app_menu)
 		gtk_signal_connect (GTK_OBJECT (menuitem), "destroy",
 				    (GtkSignalFunc) free_app_def, item_info);
 
-		activate_func = create_app_menu ? add_app_to_panel : activate_app_def;
+		activate_func = create_app_menu ? (GtkSignalFunc) add_app_to_panel : (GtkSignalFunc) activate_app_def;
 		gtk_signal_connect (GTK_OBJECT (menuitem), "activate", activate_func, item_info);
 
 		g_free (filename);
@@ -244,6 +259,8 @@ menu_position (GtkMenu *menu, gint *x, gint *y, gpointer data)
 	int wx, wy;
 	
 	gdk_window_get_origin (widget->window, &wx, &wy);
+
+	/* FIXME: This should take the panel position into account */
 	
 	*x = wx;
 	*y = wy - GTK_WIDGET (menu)->allocation.height;
@@ -252,7 +269,7 @@ menu_position (GtkMenu *menu, gint *x, gint *y, gpointer data)
 void
 activate_menu (GtkWidget *widget, void *closure)
 {
-        GtkWidget *menu = closure ;
+        GtkWidget *menu = closure;
 
 	gtk_menu_popup (GTK_MENU (menu), 0, 0, menu_position, widget, 1, 0);
 }
@@ -260,20 +277,53 @@ activate_menu (GtkWidget *widget, void *closure)
 void
 panel_configure (GtkWidget *widget, void *data)
 {
-	printf ("Panel configuration\n");
+	PanelCommand cmd;
+
+	cmd.cmd = PANEL_CMD_PROPERTIES;
+
+	(*panel_cmd_func) (&cmd);
 }
 
 void
 panel_reload (GtkWidget *widget, void *data)
 {
+	fprintf(stderr, "Panel reload not yet implemented\n"); /* FIXME */
+}
+
+static AppletItem *
+applet_item_new(char *translated, char *original_id)
+{
+	AppletItem *ai;
+
+	ai = g_new(AppletItem, 1);
+	ai->translated = translated;
+	ai->original_id = original_id;
+
+	return ai;
+}
+
+static void
+applet_item_destroy(AppletItem *ai)
+{
+	g_free(ai->translated);
+	g_free(ai->original_id);
+	g_free(ai);
+}
+
+static void
+add_applet_to_panel(GtkWidget *widget, gpointer data)
+{
+	add_to_panel(gtk_object_get_user_data(GTK_OBJECT(widget)),
+		     NULL); /* NULL means request default params */
 }
 
 static void
 munge_applet_item(gpointer untrans, gpointer user_data)
 {
-	GList **list;
-	GList  *node;
-	int     pos;
+	GList      **list;
+	GList       *node;
+	int          pos;
+	char        *trans;
 
 	list = user_data;
 
@@ -281,31 +331,46 @@ munge_applet_item(gpointer untrans, gpointer user_data)
 	
 	node = *list;
 	pos  = 0;
+
+	trans = _(untrans);
 	
 	for (pos = 0; node; node = node->next, pos++)
-		if (strcmp(_(untrans), _(node->data)) < 0)
+		if (strcmp(trans, _(node->data)) < 0)
 			break;
-	
-	*list = g_list_insert(*list, g_strdup(_(untrans)), pos);
 
-	/* We can destroy the untranslated id now */
-
-	g_free(untrans);
+	*list = g_list_insert(*list,
+			      applet_item_new(g_strdup(trans), untrans),
+			      pos);
 }
 
 static void
-append_list_item_to_menu(gpointer data, gpointer user_data)
+append_applet_item_to_menu(gpointer data, gpointer user_data)
 {
-	GtkMenu   *menu;
-	GtkWidget *menuitem;
+	GtkMenu    *menu;
+	GtkWidget  *menuitem;
+	AppletItem *ai;
+	char       *oid;
 
+	ai = data;
 	menu = GTK_MENU(user_data);
+
+	oid = g_strdup(ai->original_id);
+
 	menuitem = gtk_menu_item_new();
-	setup_menuitem(menuitem, NULL, data);
-	/* FIXME: connect to "activate" signal */
+	setup_menuitem(menuitem, NULL, ai->translated);
+	gtk_object_set_user_data(GTK_OBJECT(menuitem), oid);
+	gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
+			   (GtkSignalFunc) add_applet_to_panel,
+			   NULL);
+	gtk_signal_connect(GTK_OBJECT(menuitem), "destroy",
+			   (GtkSignalFunc) free_string,
+			   oid);
+	
 	gtk_menu_append(menu, menuitem);
 
-	g_free(data); /* We can now destroy the string */
+	/* Free applet item */
+
+	applet_item_destroy(ai);
 }
 
 static GtkWidget *
@@ -319,33 +384,30 @@ create_applets_menu(void)
 	/* Get list of applet types */
 
 	cmd.cmd = PANEL_CMD_GET_APPLET_TYPES;
-	list = (*callback) (&cmd);
+	list = (*panel_cmd_func) (&cmd);
 
 	/* Now translate and sort them */
 
 	applets_list = NULL;
 	g_list_foreach(list, munge_applet_item, &applets_list);
 
-	/* Untranslated items have been destroyed by
-	 * munge_applet_item(), now create a menu of the translated
-	 * and sorted ones.
-	 */
+	/* Create a menu of the translated and sorted ones */
 
 	g_list_free(list);
 
 	menu = gtk_menu_new();
 
-	g_list_foreach(applets_list, append_list_item_to_menu, menu);
+	g_list_foreach(applets_list, append_applet_item_to_menu, menu);
 
-	/* Translated items have been destroyed by
-	 * append_list_item_to_menu(), now just destroy the list.
+	/* Destroy the list (the list items have already been freed by
+	 * append_applet_item_to_menu()), and return the finished menu.
 	 */
 
 	g_list_free(applets_list);
 	return menu;
 }
 
-GtkWidget *
+static GtkWidget *
 create_panel_submenu (GtkWidget *app_menu)
 {
 	GtkWidget *menu, *menuitem;
@@ -377,22 +439,22 @@ create_panel_submenu (GtkWidget *app_menu)
 	return menu;
 }
 
-void
+static void
 panel_lock (GtkWidget *widget, void *data)
 {
 	system ("gnome-lock");
 }
 
-void
+static void
 panel_logout (GtkWidget *widget, void *data)
 {
 	PanelCommand cmd;
 
 	cmd.cmd = PANEL_CMD_QUIT;
-	(*callback) (&cmd);
+	(*panel_cmd_func) (&cmd);
 }
 
-void
+static void
 add_special_entries (GtkWidget *menu, GtkWidget *app_menu)
 {
 	GtkWidget *menuitem;
@@ -419,8 +481,7 @@ add_special_entries (GtkWidget *menu, GtkWidget *app_menu)
 	gtk_signal_connect (GTK_OBJECT (menuitem), "activate", (GtkSignalFunc) panel_logout, 0);
 }
 
-
-GtkWidget *
+static GtkWidget *
 create_panel_menu (GtkWidget *window, char *menudir, int main_menu)
 {
 	GtkWidget *vbox, *button, *pixmap, *button_pixmap, *menu, *app_menu;
@@ -463,10 +524,10 @@ create_panel_menu (GtkWidget *window, char *menudir, int main_menu)
 	return vbox;
 }
 
-GtkWidget *
+static GtkWidget *
 create_menu_widget (GtkWidget *window, char *arguments, char *menudir)
 {
-	GtkWidget *menu, *add_menu;
+	GtkWidget *menu;
 	int main_menu;
 	
 	main_menu = (strcmp (arguments, ".") == 0);
@@ -474,14 +535,8 @@ create_menu_widget (GtkWidget *window, char *arguments, char *menudir)
 	return menu;
 }
 
-gpointer
-panel_callback (AppletCommand *cmd)
-{
-	return NULL;
-}
-
-void
-init (PanelCallback cback, Panel *panel, char *params, int xpos, int ypos)
+static void
+create_instance (Panel *panel, char *params, int xpos, int ypos)
 {
 	GtkWidget *menu_component;
 	char *menu_base = gnome_unconditional_datadir_file ("apps");
@@ -491,8 +546,6 @@ init (PanelCallback cback, Panel *panel, char *params, int xpos, int ypos)
 	if (!getenv ("PATH"))
 		return;
 
-	fprintf (stderr, "Menu: %s\n", params);
-	
 	if (*params == '/')
 		this_menu = strdup (params);
 	else 
@@ -510,22 +563,71 @@ init (PanelCallback cback, Panel *panel, char *params, int xpos, int ypos)
 		gnome_folder = NULL;
 	}
 	
-	callback = cback;
 	menu_component = create_menu_widget (panel->window, params, this_menu);
 
-	fprintf (stderr, "REGISTRANDO: %s\n", params);
+	/* FIXME: this is a hack to remember the menu path.  It should
+	 * really be carried around by the menu's properties
+	 */
+	
+	gtk_object_set_data(GTK_OBJECT(menu_component), MENU_PATH, g_strdup(params));
+	
 	cmd.cmd = PANEL_CMD_REGISTER_TOY;
-	cmd.params.register_toy.applet   = menu_component;
-	cmd.params.register_toy.callback = panel_callback;
-	cmd.params.register_toy.id       = query();
-	cmd.params.register_toy.params   = params;
-	cmd.params.register_toy.xpos     = xpos;
-	cmd.params.register_toy.ypos     = ypos;
-	cmd.params.register_toy.flags    = APPLET_HAS_PROPERTIES;
+	cmd.params.register_toy.applet = menu_component;
+	cmd.params.register_toy.id     = APPLET_ID;
+	cmd.params.register_toy.xpos   = xpos;
+	cmd.params.register_toy.ypos   = ypos;
+	cmd.params.register_toy.flags  = APPLET_HAS_PROPERTIES;
 
-	(*callback) (&cmd);
+	(*panel_cmd_func) (&cmd);
 }
 
+
+gpointer
+applet_cmd_func(AppletCommand *cmd)
+{
+	g_assert(cmd != NULL);
+
+	switch (cmd->cmd) {
+		case APPLET_CMD_QUERY:
+			return APPLET_ID;
+
+		case APPLET_CMD_INIT_MODULE:
+			panel_cmd_func = cmd->params.init_module.cmd_func;
+			break;
+
+		case APPLET_CMD_DESTROY_MODULE:
+			break;
+
+		case APPLET_CMD_GET_DEFAULT_PARAMS:
+			return g_strdup(".");
+
+		case APPLET_CMD_CREATE_INSTANCE:
+			create_instance(cmd->panel,
+					cmd->params.create_instance.params,
+					cmd->params.create_instance.xpos,
+					cmd->params.create_instance.ypos);
+			break;
+
+		case APPLET_CMD_GET_INSTANCE_PARAMS:
+			/* FIXME: this should return the stuff from current properties */
+			
+			return g_strdup(gtk_object_get_data(GTK_OBJECT(cmd->applet), MENU_PATH));
+
+		case APPLET_CMD_PROPERTIES:
+			fprintf(stderr, "Menu properties not yet implemented\n"); /* FIXME */
+			break;
+
+		default:
+			fprintf(stderr,
+				APPLET_ID " applet_cmd_func: Oops, unknown command type %d\n",
+				(int) cmd->cmd);
+			break;
+	}
+
+	return NULL;
+}
+
+
 #if 0
 main (int argc, char *argv [])
 {
