@@ -41,7 +41,7 @@ main (int argc, char *argv [])
 	
 	gtk_container_add (GTK_CONTAINER (window), (GtkWidget *) thing);
 	gtk_widget_set_usize (window, 48, 48);
-	gtk_window_set_policy (window, 0, 0, 1);
+	gtk_window_set_policy ((GtkWindow *) window, 0, 0, 1);
 	gtk_widget_show (window);
 	gtk_widget_realize (window);
 	gtk_main ();
