@@ -1,32 +1,48 @@
+/* Gnome panel: panel functionality
+ * (C) 1997 the Free Software Foundation
+ *
+ * Authors: Federico Mena
+ *          Miguel de Icaza
+ */
+
+#ifdef HAVE_LIBINTL
 #include <libintl.h>
+#endif
 #include <string.h>
-#include "applet.h"
+#include "gnome.h"
+#include "applet_files.h"
 #include "gdkextra.h"
+#include "panel_cmds.h"
+#include "applet_cmds.h"
+#include "panel.h"
 
+#ifdef HAVE_LIBINTL
 #define _(String) gettext(String)
+#else
+#define _(String) (String)
+#endif
 
 
-#define APPLET_CALLBACK "panel_applet_callback"
-#define APPLET_ID       "panel_applet_id"
-#define APPLET_PARAMS   "panel_applet_params"
+#define APPLET_CMD_FUNC "panel_applet_cmd_func"
 #define APPLET_FLAGS    "panel_applet_flags"
 
 #define DEFAULT_STEP_SIZE 6
 #define DEFAULT_DELAY     0
 #define DEFAULT_HEIGHT    48
 
-
-typedef struct {
-	int x, y;
-} Point;
+#define APPLET_EVENT_MASK (GDK_BUTTON_PRESS_MASK |		\
+			   GDK_BUTTON_RELEASE_MASK |		\
+			   GDK_POINTER_MOTION_MASK |		\
+			   GDK_POINTER_MOTION_HINT_MASK)
 
 
 static GtkWidget *applet_menu;
 static GtkWidget *applet_menu_prop_separator;
 static GtkWidget *applet_menu_prop_item;
 
+static GdkCursor *fleur_cursor;
 
-Panel *the_panel = NULL;
+Panel *the_panel;
 
 
 static void
@@ -88,10 +104,13 @@ pop_up(void)
 {
 	int width, height;
 	int swidth, sheight;
-	
-	if (the_panel->state == PANEL_SHOWN)
+
+	if ((the_panel->state == PANEL_MOVING) ||
+	    (the_panel->state == PANEL_SHOWN))
 		return;
 
+	the_panel->state = PANEL_MOVING;
+
 	width   = the_panel->window->allocation.width;
 	height  = the_panel->window->allocation.height;
 	swidth  = gdk_screen_width();
@@ -124,9 +143,12 @@ pop_down(void)
 	int width, height;
 	int swidth, sheight;
 
-	if (the_panel->state == PANEL_HIDDEN)
+	if ((the_panel->state == PANEL_MOVING) ||
+	    (the_panel->state == PANEL_HIDDEN))
 		return;
 
+	the_panel->state = PANEL_MOVING;
+
 	width   = the_panel->window->allocation.width;
 	height  = the_panel->window->allocation.height;
 	swidth  = gdk_screen_width();
@@ -154,22 +176,6 @@ pop_down(void)
 }
 
 
-static void
-panel_block_signals(void)
-{
-	gtk_signal_handler_block(GTK_OBJECT(the_panel->window), the_panel->enter_notify_id);
-	gtk_signal_handler_block(GTK_OBJECT(the_panel->window), the_panel->leave_notify_id);
-}
-
-
-static void
-panel_unblock_signals(void)
-{
-	gtk_signal_handler_unblock(GTK_OBJECT(the_panel->window), the_panel->leave_notify_id);
-	gtk_signal_handler_unblock(GTK_OBJECT(the_panel->window), the_panel->enter_notify_id);
-}
-
-
 static gint
 panel_enter_notify(GtkWidget *widget, GdkEventCrossing *event, gpointer data)
 {
@@ -177,9 +183,7 @@ panel_enter_notify(GtkWidget *widget, GdkEventCrossing *event, gpointer data)
 	    (event->detail == GDK_NOTIFY_INFERIOR))
 		return FALSE;
 
-	panel_block_signals();
 	pop_up();
-	panel_unblock_signals();
 	
 	return FALSE;
 }
@@ -192,56 +196,11 @@ panel_leave_notify(GtkWidget *widget, GdkEventCrossing *event, gpointer data)
 	    (event->detail == GDK_NOTIFY_INFERIOR))
 		return FALSE;
 
-	panel_block_signals();
 	pop_down();
-	panel_unblock_signals();
 
 	return FALSE;
 }
 
-static void
-max_pos(GtkWidget *widget, gpointer data)
-{
-	Point *point;
-
-	point = data;
-
-	point->x = MAX(point->x, widget->allocation.x + widget->allocation.width);
-	point->y = MAX(point->y, widget->allocation.y + widget->allocation.height);
-}
-
-
-static void
-find_applet_placement(int *x, int *y)
-{
-	Point point;
-
-	point.x = 0;
-	point.y = 0;
-
-	*x = 0;
-	*y = 0;
-	return; /* FIXME: this is for now; fix to calculate appropriate position */
-	
-	gtk_container_foreach(GTK_CONTAINER(the_panel->fixed),
-			      (GtkCallback) max_pos,
-			      &point);
-
-	switch (the_panel->pos) {
-		case PANEL_POS_TOP:
-		case PANEL_POS_BOTTOM:
-			*x = point.x;
-			*y = 0;
-			break;
-
-		case PANEL_POS_LEFT:
-		case PANEL_POS_RIGHT:
-			*x = 0;
-			*y = point.y;
-			break;
-	}
-}
-
 
 static void
 change_window_cursor(GdkWindow *window, GdkCursorType cursor_type)
@@ -261,29 +220,15 @@ realize_change_cursor(GtkWidget *widget, gpointer data)
 }
 
 
-static AppletCallback
-get_applet_callback(GtkWidget *applet)
+static AppletCmdFunc
+applet_cmd_func(GtkWidget *applet)
 {
-	return gtk_object_get_data(GTK_OBJECT(applet), APPLET_CALLBACK);
-}
-
-
-static char *
-get_applet_id(GtkWidget *applet)
-{
-	return gtk_object_get_data(GTK_OBJECT(applet), APPLET_ID);
-}
-
-
-static char *
-get_applet_params(GtkWidget *applet)
-{
-	return gtk_object_get_data(GTK_OBJECT(applet), APPLET_PARAMS);
+	return gtk_object_get_data(GTK_OBJECT(applet), APPLET_CMD_FUNC);
 }
 
 
 static long
-get_applet_flags(GtkWidget *applet)
+applet_flags(GtkWidget *applet)
 {
 	return (long) gtk_object_get_data(GTK_OBJECT(applet), APPLET_FLAGS);
 }
@@ -309,53 +254,84 @@ get_applet_geometry(GtkWidget *applet, int *x, int *y, int *width, int *height)
 static gpointer
 call_applet(GtkWidget *applet, AppletCommand *cmd)
 {
-	AppletCallback callback;
+	AppletCmdFunc cmd_func;
 	
 	cmd->panel  = the_panel;
 	cmd->applet = GTK_BIN(applet)->child;
 
-	callback = get_applet_callback(applet);
+	cmd_func = applet_cmd_func(applet);
 
-	if (callback)
-		return (*callback) (cmd);
-	else
-		return NULL;
+	return (*cmd_func) (cmd);
 }
 
 
 static void
 save_applet_configuration(GtkWidget *widget, gpointer data)
 {
-	static int num = 0;
+	int           *num;
+	int            xpos, ypos;
+	char          *id;
+	char          *params;
+	char          *path;
+	char           buf[256];
+	AppletCommand  cmd;
 
-	int   xpos, ypos;
-	char *id;
-	char *params;
-	char *path;
-	char  buf[256];
+	num = data;
 
-	id     = get_applet_id(widget);
-	params = get_applet_params(widget);
+	cmd.cmd = APPLET_CMD_QUERY;
+	id      = call_applet(widget, &cmd);
+
+	cmd.cmd = APPLET_CMD_GET_INSTANCE_PARAMS;
+	params  = call_applet(widget, &cmd);
+	
 	get_applet_geometry(widget, &xpos, &ypos, NULL, NULL);
 
 	/* XXX: The increasing number is sort of a hack to guarantee unique keys */
 
-	sprintf(buf, "%d,", num++);
+	sprintf(buf, "%d,", (*num)++);
 	path = g_copy_strings("/panel/Applets/", buf, id, ",", params, NULL);
 	sprintf(buf, "%d %d", xpos, ypos);
+
 	gnome_config_set_string(path, buf);
+
+	g_free(params);
 	g_free(path);
 }
 
 
+static void
+destroy_applet_module(gpointer key, gpointer value, gpointer user_data)
+{
+	AppletCommand  cmd;
+	AppletFile    *af;
+
+	cmd.cmd    = APPLET_CMD_DESTROY_MODULE;
+	cmd.panel  = the_panel;
+	cmd.applet = NULL;
+
+	af = value;
+
+	(*af->cmd_func) (&cmd);
+}
+
+
 static void
 panel_quit(void)
 {
+	int num;
+	
+	num = 0;
 	gnome_config_clean_section("/panel/Applets");
-	gtk_container_foreach(GTK_CONTAINER(the_panel->fixed), save_applet_configuration, NULL);
+	gtk_container_foreach(GTK_CONTAINER(the_panel->fixed), save_applet_configuration, &num);
 	gnome_config_sync();
+
+	gdk_cursor_destroy(fleur_cursor);
+	
 	gtk_widget_destroy(the_panel->window);
-	applets_destroy();
+
+	g_hash_table_foreach(applet_files_ht, destroy_applet_module, NULL);
+	
+	applet_files_destroy();
 }
 
 
@@ -380,6 +356,12 @@ applet_drag_start(GtkWidget *applet, int warp)
 			    NULL);
 
 	gtk_grab_add(applet);
+	gdk_pointer_grab(applet->window,
+			 TRUE,
+			 APPLET_EVENT_MASK,
+			 NULL,
+			 fleur_cursor,
+			 GDK_CURRENT_TIME);
 }
 
 
@@ -387,6 +369,8 @@ static void
 applet_drag_end(GtkWidget *applet)
 {
 	the_panel->applet_being_dragged = NULL;
+
+	gdk_pointer_ungrab(GDK_CURRENT_TIME);
 	gtk_grab_remove(applet);
 }
 
@@ -465,10 +449,6 @@ create_applet_menu(void)
 	gtk_menu_append(GTK_MENU(applet_menu), menuitem);
 	gtk_widget_show(menuitem);
 	applet_menu_prop_item = menuitem;
-
-	gtk_signal_connect_after(GTK_OBJECT(applet_menu), "deactivate",
-				 (GtkSignalFunc) panel_unblock_signals,
-				 NULL);
 }
 
 
@@ -477,7 +457,7 @@ show_applet_menu(GtkWidget *applet)
 {
 	long flags;
 
-	flags = get_applet_flags(applet);
+	flags = applet_flags(applet);
 
 	if (flags & APPLET_HAS_PROPERTIES) {
 		gtk_widget_show(applet_menu_prop_separator);
@@ -489,7 +469,6 @@ show_applet_menu(GtkWidget *applet)
 
 	gtk_object_set_user_data(GTK_OBJECT(applet_menu), applet);
 
-	panel_block_signals();
 	gtk_menu_popup(GTK_MENU(applet_menu), NULL, NULL, NULL, NULL, 3, 0);
 }
 
@@ -577,6 +556,7 @@ panel_applet_event(GtkWidget *widget, GdkEvent *event, gpointer data)
 	return FALSE;
 }
 
+
 static GtkWidget *
 listening_parent(GtkWidget *widget)
 {
@@ -588,7 +568,7 @@ listening_parent(GtkWidget *widget)
 
 
 static gint
-panel_sub_event_handler (GtkWidget *widget, GdkEvent *event, gpointer data)
+panel_sub_event_handler(GtkWidget *widget, GdkEvent *event, gpointer data)
 {
 	GdkEventButton *bevent;
 
@@ -659,7 +639,6 @@ panel_init(void)
 	the_panel->mode  = PANEL_GETS_HIDDEN;
 	gtk_widget_set_usize(the_panel->window, gdk_screen_width(), DEFAULT_HEIGHT);
 	gtk_widget_set_uposition(the_panel->window, 0, gdk_screen_height() - DEFAULT_HEIGHT);
-
 #if 0
 	the_panel->pos   = PANEL_POS_TOP;
 	the_panel->state = PANEL_SHOWN;
@@ -695,100 +674,281 @@ panel_init(void)
 				 (GtkSignalFunc) realize_change_cursor,
 				 NULL);
 
+	fleur_cursor = gdk_cursor_new(GDK_FLEUR);
+
 	create_applet_menu();
 }
 
 
-void
-panel_bind_events (GtkWidget *widget, void *data)
+static void
+init_applet_module(gpointer key, gpointer value, gpointer user_data)
 {
-	gint       events;
+	AppletCommand  cmd;
+	AppletFile    *af;
 
-	if (!GTK_WIDGET_NO_WINDOW (widget)) {
-		events = (gtk_widget_get_events(widget) |
-			  GDK_BUTTON_PRESS_MASK |
-			  GDK_BUTTON_RELEASE_MASK |
-			  GDK_POINTER_MOTION_MASK |
-			  GDK_POINTER_MOTION_HINT_MASK);
-		
-		gtk_widget_set_events(widget, events);
-		
-		gtk_signal_connect(GTK_OBJECT(widget), "event",
-				   (GtkSignalFunc) panel_sub_event_handler, the_panel);
-	}
-	
-	if (GTK_IS_CONTAINER (widget))
-		gtk_container_foreach (GTK_CONTAINER (widget), panel_bind_events, 0);
+	cmd.cmd    = APPLET_CMD_INIT_MODULE;
+	cmd.panel  = the_panel;
+	cmd.applet = NULL;
+	cmd.params.init_module.cmd_func = panel_command;
+
+	af = value;
+
+	(*af->cmd_func) (&cmd);
 }
 
 
 void
-panel_register_toy(GtkWidget *applet, char *id, char *params, int xpos, int ypos, long flags)
+panel_init_applet_modules(void)
+{
+	g_hash_table_foreach(applet_files_ht, init_applet_module, NULL);
+}
+
+
+static void
+create_applet(char *id, char *params, int xpos, int ypos)
+{
+	AppletCommand cmd;
+	AppletCmdFunc cmd_func;
+	int           requested;
+	
+	g_assert(id != NULL);
+	
+	cmd_func = get_applet_cmd_func(id);
+	if (!cmd_func) {
+		fprintf(stderr, "create_applet: could not find applet \"%s\"\n", id);
+		return;
+	}
+
+	requested = FALSE;
+	
+	if (!params) {
+		cmd.cmd    = APPLET_CMD_GET_DEFAULT_PARAMS;
+		cmd.panel  = the_panel;
+		cmd.applet = NULL;
+		
+		params = (*cmd_func) (&cmd);
+
+		if (!params) {
+			fprintf(stderr,
+				"create_applet: warning: applet \"%s\" returned NULL default parameters\n"
+				"               using empty parameter string \"\"\n",
+				id);
+			params = g_strdup("");
+		}
+
+		requested = TRUE;
+	}
+
+	cmd.cmd    = APPLET_CMD_CREATE_INSTANCE;
+	cmd.panel  = the_panel;
+	cmd.applet = NULL;
+	cmd.params.create_instance.params = params;
+	cmd.params.create_instance.xpos   = xpos;
+	cmd.params.create_instance.ypos   = ypos;
+
+	(*cmd_func) (&cmd);
+
+	if (requested)
+		g_free(params);
+}
+
+
+static void
+bind_applet_events(GtkWidget *widget, void *data)
+{
+	/* XXX: This is more or less a hack.  We need to be able to
+	 * capture events over applets so that we can drag them with
+	 * the mouse and such.  So we need to force the applet's
+	 * widgets to recursively send the events back to their parent
+	 * until the event gets to the applet wrapper (the
+	 * GtkEventBox) for processing by us.
+	 */
+	
+	if (!GTK_WIDGET_NO_WINDOW(widget)) {
+		gtk_widget_set_events(widget, gtk_widget_get_events(widget) | APPLET_EVENT_MASK);
+		gtk_signal_connect(GTK_OBJECT(widget), "event",
+				   (GtkSignalFunc) panel_sub_event_handler,
+				   NULL);
+	}
+	
+	if (GTK_IS_CONTAINER(widget))
+		gtk_container_foreach (GTK_CONTAINER (widget), bind_applet_events, 0);
+}
+
+
+static void
+sort_applet_by_pos(GtkWidget *applet, gpointer data)
+{
+	GList     **list;
+	GtkWidget *w;
+	int        pos;
+
+	list = data;
+	pos  = 0;
+
+	while (*list) {
+		w = (*list)->data;
+		
+		if (applet->allocation.x < w->allocation.x)
+			break;
+
+		pos++;
+		*list = (*list)->next;
+	}
+
+	*list = g_list_insert(*list, applet, pos);
+}
+
+static void
+fix_applet_position(GtkWidget *applet, int *xpos, int *ypos)
+{
+	GList         *applets;
+	GtkWidget     *w;
+	GtkRequisition requisition;
+	int            req_width;
+	int            found;
+	int            x;
+	int            size;
+	int            largest;
+	int            x_for_largest;
+	
+	/* FIXME!!!  This is not working at all --- the widgets have
+         * no allocation when I need them to :-(
+	 */
+
+	/* FIXME: This routine currently only handles the horizontal case of the panel */
+	
+	if ((*xpos != PANEL_UNKNOWN_APPLET_POSITION) &&
+	    (*ypos != PANEL_UNKNOWN_APPLET_POSITION))
+		return;
+
+	*ypos = 0;
+
+	applets = NULL;
+	gtk_container_foreach(GTK_CONTAINER(the_panel->fixed),
+			      sort_applet_by_pos,
+			      &applets);
+
+	gtk_widget_size_request(applet, &requisition);
+	req_width = requisition.width;
+
+	/* Find first fit */
+
+	found         = FALSE;
+	x             = 0;
+	largest       = 0;
+	x_for_largest = 0;
+
+	for (; applets; applets = applets->next) {
+		w = applets->data;
+
+		size = w->allocation.x - x;
+
+		if (size >= req_width) {
+			found = TRUE;
+			break;
+		}
+
+		if (size > largest) {
+			largest = size;
+			x_for_largest = x;
+		}
+
+		x = w->allocation.x + w->allocation.width;
+	}
+
+	if (found)
+		*xpos = x;
+	else {
+		size = gdk_screen_width() - x;
+
+		if (size >= req_width)
+			*xpos = x;
+		else
+			*xpos = x_for_largest;
+	}
+
+	g_list_free(applets);
+}
+
+
+static void
+register_toy(GtkWidget *applet, char *id, int xpos, int ypos, long flags)
 {
 	GtkWidget *eventbox;
-	gint      events;
 	
 	g_assert(applet != NULL);
+	g_assert(id != NULL);
 
 	/* We wrap the applet in a GtkEventBox so that we can capture events over it */
 
 	eventbox = gtk_event_box_new();
 	gtk_container_add(GTK_CONTAINER(eventbox), applet);
 
-	events = (gtk_widget_get_events(eventbox) |
-		  GDK_BUTTON_PRESS_MASK |
-		  GDK_BUTTON_RELEASE_MASK |
-		  GDK_POINTER_MOTION_MASK |
-		  GDK_POINTER_MOTION_HINT_MASK);
-	
-	gtk_widget_set_events(eventbox, events);
-	
-	gtk_signal_connect(GTK_OBJECT(eventbox), "event", (GtkSignalFunc) panel_applet_event, the_panel);
-	panel_bind_events (applet, 0);
+	gtk_widget_set_events(eventbox, gtk_widget_get_events(eventbox) | APPLET_EVENT_MASK);
+	gtk_signal_connect(GTK_OBJECT(eventbox), "event", (GtkSignalFunc) panel_applet_event, NULL);
+	bind_applet_events(applet, NULL);
 	
 	/* Attach our private data to the applet */
 
-	gtk_object_set_data(GTK_OBJECT(eventbox), APPLET_ID, id);
-	gtk_object_set_data(GTK_OBJECT(eventbox), APPLET_PARAMS, params);
+	gtk_object_set_data(GTK_OBJECT(eventbox), APPLET_CMD_FUNC, get_applet_cmd_func(id));
 	gtk_object_set_data(GTK_OBJECT(eventbox), APPLET_FLAGS, (gpointer) flags);
 
+	fix_applet_position(eventbox, &xpos, &ypos);
+
 	gtk_fixed_put(GTK_FIXED(the_panel->fixed), eventbox, xpos, ypos);
 	gtk_widget_show(eventbox);
 	gtk_widget_show(applet);
 }
 
 
+static void
+properties(void)
+{
+	fprintf(stderr, "Panel properties not yet implemented\n");
+}
+
+
 gpointer
 panel_command(PanelCommand *cmd)
 {
-	int xpos, ypos;
-	
 	g_assert(cmd != NULL);
 	
 	switch (cmd->cmd) {
-		case PANEL_CMD_REGISTER_TOY:
-			panel_register_toy(cmd->params.register_toy.applet,
-					   cmd->params.register_toy.id,
-					   cmd->params.register_toy.params,
-					   cmd->params.register_toy.xpos,
-					   cmd->params.register_toy.ypos,
-					   cmd->params.register_toy.flags);
-			return NULL;
-
 		case PANEL_CMD_QUIT:
 			panel_quit();
 			gtk_main_quit();
 			return NULL;
 
-		case PANEL_CMD_NEW_APPLET:
-			find_applet_placement(&xpos, &ypos);
-			applets_init_applet(cmd->params.new_applet.applet,
-					    cmd->params.new_applet.params,
-					    xpos, ypos);
-			return NULL;
-
 		case PANEL_CMD_GET_APPLET_TYPES:
 			return get_applet_types();
+
+		case PANEL_CMD_GET_APPLET_CMD_FUNC:
+			return get_applet_cmd_func(cmd->params.get_applet_cmd_func.id);
+
+		case PANEL_CMD_CREATE_APPLET:
+			create_applet(cmd->params.create_applet.id,
+				      cmd->params.create_applet.params,
+				      cmd->params.create_applet.xpos,
+				      cmd->params.create_applet.ypos);
+			break;
+			
+		case PANEL_CMD_REGISTER_TOY:
+			register_toy(cmd->params.register_toy.applet,
+				     cmd->params.register_toy.id,
+				     cmd->params.register_toy.xpos,
+				     cmd->params.register_toy.ypos,
+				     cmd->params.register_toy.flags);
+			break;
+
+		case PANEL_CMD_PROPERTIES:
+			properties();
+			break;
+
+		default:
+			fprintf(stderr, "panel_command: Oops, unknown command type %d\n",
+				(int) cmd->cmd);
+			break;
 	}
 
 	return NULL;
